function [ f_QCQP, f_mod1_QCQP, err_f_QCQP, err_f_mod1_QCQP, f_mod1_QCQP_raw, f_mod1_QCQP_OLSnoShift, runningTime_TRS ] ...
    = qcqp_solver( f_clean, f_mod1_clean, f_noise, f_mod1_noise, ...
    k, sigma, x_grid, DATA ,PARS, to_plot, lambda )

%-------------------------------------------------------------
% Graph: G = (V,E) where (i,j) is in E if |i-j| <= k
% Create Adjacency matrix (Adj) and Laplacian matrix (L) of G.
%--------------------------------------------------------------

n = length(f_noise);
distance_type = 'chebychev';
if PARS.dim == 1
    Coord = x_grid';
else
    Coord = x_grid;
end
kDist = k;
targetDegree = (2*kDist + 1)^PARS.dim - 1;
G = build_KNN_DIST_ADJ_subset(Coord, targetDegree, 1:n, distance_type);
% full(G);
Adj = G;

L = diag(sum(Adj,2)) - Adj; % Laplacian matrix of G
Hbar = blkdiag(lambda*L, lambda*L); % 2n x 2n block diagonal matrix

%--------------------------------------------------------------------------
% The following code denoises the mod 1 function values via a QCQP with
% quadratic equality constraints.
% The output is:
%       nx1 vector: y_denoise (denoised mod 1 values)
%--------------------------------------------------------------------------
tic
[ f_mod1_QCQP ] = qcqp_alg(f_mod1_noise, Hbar);
runningTime_TRS = toc;

f_mod1_QCQP_raw = f_mod1_QCQP;

to_plotInner= 0;
[ f_QCQP, f_mod1_QCQP, err_f_QCQP, ~ , f_est_noShift_OLS] = ...
    ols_solver( f_clean, f_mod1_clean, ...
    f_noise, f_mod1_QCQP, k, sigma, x_grid, DATA ,PARS, to_plotInner);

f_mod1_QCQP_OLSnoShift = mod(f_est_noShift_OLS, 1);

err = getErrors(f_mod1_clean, f_mod1_QCQP_raw);
err_f_mod1_QCQP = err(1);

end

